@@ -8,14 +8,22 @@
 
 #include "camel-exception.h"
 
+typedef int(*CamelDBCollate)(void*,int,const void*,int,const void*);
+
 struct _CamelDB {
 	sqlite3 *db;
 	GMutex *lock;
+	const char *sort_by;
+	const char *collate;
+	CamelDBCollate collate_cb;
+#if CAMEL_DB_DEBUG 	
+	GTimer *timer;
+#endif	
 };
 
 #define CAMEL_DB_FREE_CACHE_SIZE 2 * 1024 * 1024
 #define CAMEL_DB_SLEEP_INTERVAL 1*10*10
-#define CAMEL_DB_RELEASE_SQLITE_MEMORY if(g_getenv("CAMEL_SQLITE_FREE_CACHE")) sqlite3_release_memory(CAMEL_DB_FREE_CACHE_SIZE);
+#define CAMEL_DB_RELEASE_SQLITE_MEMORY if(!g_getenv("CAMEL_SQLITE_FREE_CACHE")) sqlite3_release_memory(CAMEL_DB_FREE_CACHE_SIZE);
 #define CAMEL_DB_USE_SHARED_CACHE if(g_getenv("CAMEL_SQLITE_SHARED_CACHE")) sqlite3_enable_shared_cache(TRUE);
 
 
@@ -109,6 +117,7 @@ int camel_db_add_to_transaction (CamelDB *cdb, const char *query, CamelException
 int camel_db_end_transaction (CamelDB *cdb, CamelException *ex);
 int camel_db_abort_transaction (CamelDB *cdb, CamelException *ex);
 int camel_db_clear_folder_summary (CamelDB *cdb, char *folder, CamelException *ex);
+int camel_db_rename_folder (CamelDB *cdb, const char *old_folder, const char *new_folder, CamelException *ex);
 
 int camel_db_delete_folder (CamelDB *cdb, const char *folder, CamelException *ex);
 int camel_db_delete_uid (CamelDB *cdb, const char *folder, const char *uid, CamelException *ex);
@@ -140,7 +149,9 @@ int camel_db_count_junk_not_deleted_message_info (CamelDB *cdb, const char *tabl
 void camel_db_camel_mir_free (CamelMIRecord *record);
 
 int camel_db_create_vfolder (CamelDB *db, const char *folder_name, CamelException *ex);
+int camel_db_recreate_vfolder (CamelDB *db, const char *folder_name, CamelException *ex);
 int camel_db_delete_uid_from_vfolder (CamelDB *db, char *folder_name, char *vuid, CamelException *ex);
+int camel_db_delete_uid_from_vfolder_transaction (CamelDB *db, char *folder_name, char *vuid, CamelException *ex);
 GPtrArray * camel_db_get_vuids_from_vfolder (CamelDB *db, char *folder_name, char *filter, CamelException *ex);
 int camel_db_add_to_vfolder (CamelDB *db, char *folder_name, char *vuid, CamelException *ex);
 int camel_db_add_to_vfolder_transaction (CamelDB *db, char *folder_name, char *vuid, CamelException *ex);
@@ -154,6 +165,8 @@ char * camel_db_sqlize_string (const char *string);
 void camel_db_free_sqlized_string (char *string);
 
 char * camel_db_get_column_name (const char *raw_name);
-
+int camel_db_set_collate (CamelDB *cdb, const char *col, const char *collate, CamelDBCollate func);
+/* Migration APIS */
+int camel_db_migrate_vfolders_to_14(CamelDB *cdb, const char *folder, CamelException *ex);
 #endif
 
