@@ -44,6 +44,7 @@
 #include "camel-vee-store.h"	/* for open flags */
 #include "camel-vee-summary.h"
 #include "camel-string-utils.h"
+#include "camel-vee-folder.h"
 
 #define d(x) 
 #define dd(x) (camel_debug("vfolder")?(x):0)
@@ -537,7 +538,7 @@ summary_header_to_db (CamelFolderSummary *s, CamelException *ex)
 	record->time = s->time;
 
 	record->saved_count = s->uids->len;
-	if ((s->unread_count || s->visible_count) && !g_getenv("FORCE_VFOLDER_COUNT")) {
+	if ((s->visible_count) && !g_getenv("FORCE_VFOLDER_COUNT")) {
 		/* We should be in sync always. so use the count. Don't search.*/
 		record->junk_count = s->junk_count;
 		record->deleted_count = s->deleted_count;
@@ -578,14 +579,17 @@ vee_sync(CamelFolder *folder, gboolean expunge, CamelException *ex)
 		CamelFolder *f = node->data;
 
 		camel_folder_sync(f, expunge, ex);
-		if (camel_exception_is_set(ex)) {
+		if (camel_exception_is_set(ex) && strncmp(camel_exception_get_description(ex), "no such table", 13)) {
 			char *desc;
 
 			camel_object_get(f, NULL, CAMEL_OBJECT_DESCRIPTION, &desc, NULL);
 			camel_exception_setv(ex, ex->id, _("Error storing '%s': %s"), desc, ex->desc);
 			g_warning ("%s", camel_exception_get_description(ex));
-		}
+			g_free(desc);
+		} else
+			camel_exception_clear (ex);
 
+		
 		/* auto update vfolders shouldn't need a rebuild */
 /* 		if ((vf->flags & CAMEL_STORE_VEE_FOLDER_AUTO) == 0 */
 /* 		    && camel_vee_folder_rebuild_folder(vf, f, ex) == -1) */
@@ -593,11 +597,14 @@ vee_sync(CamelFolder *folder, gboolean expunge, CamelException *ex)
 
 		node = node->next;
 	}
-	
+#if 0
+	/* Seems like we are doing something wrong with this, as folder_changed happens after this, the counts are misleading.
+	 * Anyways we do a force sync on exit, it should be all fine.
+	  */
 	record = summary_header_to_db (folder->summary, ex);
 	camel_db_write_folder_info_record (folder->parent_store->cdb, record, ex);
 	g_free (record);
-	
+#endif	
 	if (node == NULL) {
 		CAMEL_VEE_FOLDER_LOCK(vf, changed_lock);
 		g_list_free(p->folders_changed);
@@ -613,7 +620,9 @@ vee_sync(CamelFolder *folder, gboolean expunge, CamelException *ex)
 static void
 vee_expunge (CamelFolder *folder, CamelException *ex)
 {
+	/* Force it to rebuild the counts, when some folders were expunged. */
 	folder->summary->unread_count = 0;
+	folder->summary->visible_count = 0;
 	((CamelFolderClass *)((CamelObject *)folder)->klass)->sync(folder, TRUE, ex);
 }
 
@@ -786,8 +795,9 @@ static void vee_delete(CamelFolder *folder)
 		CAMEL_VEE_FOLDER_LOCK(folder, subfolder_lock);
 	}
 	CAMEL_VEE_FOLDER_UNLOCK(folder, subfolder_lock);
-
+	
 	((CamelFolderClass *)camel_vee_folder_parent)->delete(folder);
+	((CamelVeeFolder *)folder)->deleted = TRUE;
 }
 
 /* ********************************************************************** *
@@ -1051,11 +1061,11 @@ vee_rebuild_folder(CamelVeeFolder *vf, CamelFolder *source, CamelException *ex)
 				return 0;
 			rebuilded = TRUE;
 			/* Update the counts */
-			COUNT_ADD_EXPR(f->summary->junk_count, "(match-all (system-flag  \"junk\"))");
-			COUNT_ADD_EXPR(f->summary->deleted_count, "(match-all (system-flag  \"deleted\"))");
-			COUNT_ADD_EXPR(f->summary->unread_count, "(match-all (not (system-flag  \"Seen\")))");
-			COUNT_ADD_EXPR(f->summary->visible_count, "(match-all (and (not (system-flag \"deleted\")) (not (system-flag \"junk\"))))");
-			COUNT_ADD_EXPR(f->summary->junk_not_deleted_count, "(match-all (and (not (system-flag \"deleted\")) (system-flag \"junk\")))");
+			COUNT_ADD_EXPR(folder->summary->junk_count, "(match-all (system-flag  \"junk\"))");
+			COUNT_ADD_EXPR(folder->summary->deleted_count, "(match-all (system-flag  \"deleted\"))");
+			COUNT_ADD_EXPR(folder->summary->unread_count, "(match-all (not (system-flag  \"Seen\")))");
+			COUNT_ADD_EXPR(folder->summary->visible_count, "(match-all (and (not (system-flag \"deleted\")) (not (system-flag \"junk\"))))");
+			COUNT_ADD_EXPR(folder->summary->junk_not_deleted_count, "(match-all (and (not (system-flag \"deleted\")) (system-flag \"junk\")))");
 		}
 		
 	}
@@ -2042,6 +2052,7 @@ camel_vee_folder_init (CamelVeeFolder *obj)
 	obj->search = camel_folder_search_new();
 	obj->hashes = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 	obj->loaded = g_hash_table_new (g_direct_hash, g_direct_equal);
+	obj->deleted = FALSE;
 	p->summary_lock = g_mutex_new();
 	p->subfolder_lock = g_mutex_new();
 	p->changed_lock = g_mutex_new();
@@ -2121,6 +2132,17 @@ vee_folder_stop_folder(CamelVeeFolder *vf, CamelFolder *sub)
 	camel_object_unref((CamelObject *)sub);
 }
 
+void
+camel_vee_folder_sync_headers (CamelVeeFolder *vf, CamelException *ex)
+{
+	CamelFIRecord * record;
+
+	/* Save the counts to DB */
+	record = summary_header_to_db (((CamelFolder *)vf)->summary, ex);
+	camel_db_write_folder_info_record (((CamelFolder *) vf)->parent_store->cdb, record, ex);
+	g_free (record);
+}
+
 static void
 camel_vee_folder_finalise (CamelObject *obj)
 {
@@ -2133,9 +2155,11 @@ camel_vee_folder_finalise (CamelObject *obj)
 	p->destroyed = TRUE;
 
 	/* Save the counts to DB */
-	record = summary_header_to_db (((CamelFolder *)vf)->summary, NULL);
-	camel_db_write_folder_info_record (((CamelFolder *) vf)->parent_store->cdb, record, NULL);
-	g_free (record);
+	if (!vf->deleted) {
+		record = summary_header_to_db (((CamelFolder *)vf)->summary, NULL);
+		camel_db_write_folder_info_record (((CamelFolder *) vf)->parent_store->cdb, record, NULL);
+		g_free (record);
+	}
 	
 	/* This may invoke sub-classes with partially destroyed state, they must deal with this */
 	if (vf == folder_unmatched) {
