@@ -32,6 +32,7 @@
 #include "camel-folder.h"
 #include "camel-store.h"
 #include "camel-vee-summary.h"
+#include "camel-vee-folder.h"
 #include "camel-private.h"
 #include "camel-string-utils.h"
 
@@ -57,7 +58,6 @@ vee_message_info_clone(CamelFolderSummary *s, const CamelMessageInfo *mi)
 	to = (CamelVeeMessageInfo *)camel_message_info_new(s);
 
 	to->summary = from->summary;
-	/* FIXME: We may not need this during CamelDBSummary */
 	camel_object_ref (to->summary);
 	to->info.summary = s;
 	to->info.uid = camel_pstring_strdup(from->info.uid);
@@ -65,16 +65,19 @@ vee_message_info_clone(CamelFolderSummary *s, const CamelMessageInfo *mi)
 	return (CamelMessageInfo *)to;
 }
 
+#define HANDLE_NULL_INFO(value) if (!rmi) { g_warning (G_STRLOC ": real info is NULL for %s, safeguarding\n", mi->uid); return value; }
+
 static const void *
 vee_info_ptr (const CamelMessageInfo *mi, int id)
 {
 	CamelVeeMessageInfo *vmi = (CamelVeeMessageInfo *) mi;
-	CamelMessageInfo *info;
+	CamelMessageInfo *rmi;
 	gpointer p;
 	
-	info = camel_folder_summary_uid (vmi->summary, mi->uid+8);
-	p = (gpointer) camel_message_info_ptr(info, id);
-	camel_message_info_free (info);
+	rmi = camel_folder_summary_uid (vmi->summary, mi->uid+8);
+	HANDLE_NULL_INFO(NULL);
+	p = (gpointer) camel_message_info_ptr(rmi, id);
+	camel_message_info_free (rmi);
 
 	return p;
 }
@@ -83,8 +86,12 @@ static guint32
 vee_info_uint32(const CamelMessageInfo *mi, int id)
 {
 	CamelMessageInfo *rmi = camel_folder_summary_uid (((CamelVeeMessageInfo *)mi)->summary, mi->uid+8);
-	guint32 ret = camel_message_info_uint32 (rmi, id);
-
+	guint32 ret;
+	
+	HANDLE_NULL_INFO(0);
+	ret = camel_message_info_uint32 (rmi, id);
+	if (id == CAMEL_MESSAGE_INFO_FLAGS)
+		((CamelVeeMessageInfo *) mi)->old_flags = camel_message_info_flags (rmi);
 	camel_message_info_free (rmi);
 
 	return ret;
@@ -95,7 +102,10 @@ static time_t
 vee_info_time(const CamelMessageInfo *mi, int id)
 {
 	CamelMessageInfo *rmi = camel_folder_summary_uid (((CamelVeeMessageInfo *)mi)->summary, mi->uid+8);
-	time_t ret = camel_message_info_time (rmi, id);
+	time_t ret;
+	
+	HANDLE_NULL_INFO(0);
+	ret = camel_message_info_time (rmi, id);
 	camel_message_info_free (rmi);
 
 	return ret;
@@ -105,7 +115,10 @@ static gboolean
 vee_info_user_flag(const CamelMessageInfo *mi, const char *id)
 {
 	CamelMessageInfo *rmi = camel_folder_summary_uid (((CamelVeeMessageInfo *)mi)->summary, mi->uid+8);
-	gboolean ret = 	camel_message_info_user_flag (rmi, id);
+	gboolean ret;
+	
+	HANDLE_NULL_INFO(FALSE);
+	ret = 	camel_message_info_user_flag (rmi, id);
 	camel_message_info_free (rmi);
 
 	return ret;
@@ -115,7 +128,10 @@ static const char *
 vee_info_user_tag(const CamelMessageInfo *mi, const char *id)
 {
 	CamelMessageInfo *rmi = camel_folder_summary_uid (((CamelVeeMessageInfo *)mi)->summary, mi->uid+8);
-	const char *ret = camel_message_info_user_tag (rmi, id);
+	const char *ret;
+	
+	HANDLE_NULL_INFO("");
+	ret = camel_message_info_user_tag (rmi, id);
 	camel_message_info_free (rmi);
 
 	return ret;
@@ -128,6 +144,7 @@ vee_info_set_user_flag(CamelMessageInfo *mi, const char *name, gboolean value)
 
 	if (mi->uid) {
 		CamelMessageInfo *rmi = camel_folder_summary_uid (((CamelVeeMessageInfo *)mi)->summary, mi->uid+8);
+		HANDLE_NULL_INFO(FALSE);
 		res = camel_message_info_set_user_flag(rmi, name, value);
 		camel_message_info_free (rmi);		
 	}
@@ -142,6 +159,7 @@ vee_info_set_user_tag(CamelMessageInfo *mi, const char *name, const char *value)
 
 	if (mi->uid) {
 		CamelMessageInfo *rmi = camel_folder_summary_uid (((CamelVeeMessageInfo *)mi)->summary, mi->uid+8);
+		HANDLE_NULL_INFO(FALSE);		
 		res = camel_message_info_set_user_tag(rmi, name, value);
 		camel_message_info_free (rmi);			
 	}
@@ -155,8 +173,33 @@ vee_info_set_flags(CamelMessageInfo *mi, guint32 flags, guint32 set)
 	int res = FALSE;
 
 	if (mi->uid) {
-		CamelMessageInfo *rmi = camel_folder_summary_uid (((CamelVeeMessageInfo *)mi)->summary, mi->uid+8);		
+		guint32 old_visible, old_unread, old_deleted, old_junked, old_junked_not_deleted;
+		guint32 visible, unread, deleted, junked, junked_not_deleted;
+		CamelMessageInfo *rmi = camel_folder_summary_uid (((CamelVeeMessageInfo *)mi)->summary, mi->uid+8);
+
+		HANDLE_NULL_INFO(FALSE);
+		camel_object_get(rmi->summary->folder, NULL,
+				 CAMEL_FOLDER_DELETED, &old_deleted,
+				 CAMEL_FOLDER_VISIBLE, &old_visible,
+				 CAMEL_FOLDER_JUNKED, &old_junked,
+				 CAMEL_FOLDER_JUNKED_NOT_DELETED, &old_junked_not_deleted,
+				 CAMEL_FOLDER_UNREAD, &old_unread, NULL);		
 		res = camel_message_info_set_flags(rmi, flags, set);
+		((CamelVeeMessageInfo *) mi)->old_flags = camel_message_info_flags (rmi);
+		camel_object_get(rmi->summary->folder, NULL,
+				 CAMEL_FOLDER_DELETED, &deleted,
+				 CAMEL_FOLDER_VISIBLE, &visible,
+				 CAMEL_FOLDER_JUNKED, &junked,
+				 CAMEL_FOLDER_JUNKED_NOT_DELETED, &junked_not_deleted,
+				 CAMEL_FOLDER_UNREAD, &unread, NULL);
+		/* Keep the summary in sync */
+		mi->summary->unread_count += unread - old_unread;
+		mi->summary->deleted_count += deleted - old_deleted;
+		mi->summary->junk_count += junked - old_junked;
+		mi->summary->junk_not_deleted_count += junked_not_deleted - old_junked_not_deleted;
+		mi->summary->visible_count += visible - old_visible;
+
+		d(printf("VF %d %d %d %d %d\n", mi->summary->unread_count, mi->summary->deleted_count, mi->summary->junk_count, mi->summary->junk_not_deleted_count, mi->summary->visible_count));
 		camel_message_info_free (rmi);
 	}
  
@@ -166,7 +209,7 @@ vee_info_set_flags(CamelMessageInfo *mi, guint32 flags, guint32 set)
 static CamelMessageInfo *
 message_info_from_uid (CamelFolderSummary *s, const char *uid)
 {
-	CamelMessageInfoBase *info;
+	CamelMessageInfo *info;
 
 	#warning "too bad design. Need to peek it from cfs instead of hacking ugly like this"
 	CAMEL_SUMMARY_LOCK(s, summary_lock);
@@ -179,8 +222,34 @@ message_info_from_uid (CamelFolderSummary *s, const char *uid)
 
 	CAMEL_SUMMARY_UNLOCK(s, ref_lock);
 	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
+	
+	if (!info) {
+		CamelVeeMessageInfo *vinfo;
+		char tmphash[9];
 
-	return (CamelMessageInfo *) info;	
+		/* This function isn't really nice. But no great way
+		 * But in vfolder case, this may not be so bad, as vuid has the hash in first 8 bytes.
+		 * So this just compares the entire string only if it belongs to the same folder.
+		 * Otherwise, the first byte itself would return in strcmp, saving the CPU.
+		 */
+		if (!camel_folder_summary_check_uid (s, uid)) {
+			d(g_message ("Unable to find %s in the summary of %s", uid, s->folder->full_name));
+			return NULL;
+		}
+		
+		/* Create the info and load it, its so easy. */
+		info = camel_message_info_new (s);
+		camel_message_info_ref(info);
+		info->dirty = FALSE;
+		vinfo = (CamelVeeMessageInfo *) info;
+		info->uid = camel_pstring_strdup(uid);
+		strncpy(tmphash, uid, 8);
+		tmphash[8] = 0;
+		vinfo->summary = g_hash_table_lookup(((CamelVeeFolder *) s->folder)->hashes, tmphash);
+		camel_object_ref (vinfo->summary);
+		camel_folder_summary_insert (s, info, FALSE);
+	}
+	return info;	
 }
 
 static void
@@ -238,6 +307,7 @@ camel_vee_summary_get_type (void)
 	return type;
 }
 
+
 /**
  * camel_vee_summary_new:
  * @parent: Folder its attached to.
@@ -260,7 +330,7 @@ camel_vee_summary_new(CamelFolder *parent)
 	camel_db_create_vfolder (parent->cdb, parent->full_name, NULL);
 
 	#warning "handle excep and ret"
-	camel_folder_summary_header_load_from_db ((CamelFolderSummary *)s, parent->parent_store, parent->full_name, NULL);	
+	camel_folder_summary_header_load_from_db ((CamelFolderSummary *)s, parent->parent_store, parent->full_name, NULL);
 	return &s->summary;
 }
 
@@ -288,29 +358,37 @@ camel_vee_summary_add(CamelVeeSummary *s, CamelFolderSummary *summary, const cha
 	vuid = g_malloc(strlen(uid)+9);
 	memcpy(vuid, hash, 8);
 	strcpy(vuid+8, uid);
-	
-	#warning do we need it really ?
-/* 	mi = (CamelVeeMessageInfo *)camel_folder_summary_uid(&s->summary, vuid); */
-/* 	if (mi) { */
-/* 		d(printf("w:clash, we already have '%s' in summary\n", vuid)); */
-/* 		camel_message_info_free((CamelMessageInfo *)mi); */
-/* 		g_free(vuid); */
-/* 		return NULL; */
-/* 	} */
 
-	mi = (CamelVeeMessageInfo *) message_info_from_uid(&s->summary, vuid); 
+	CAMEL_SUMMARY_LOCK(s, summary_lock);
+	mi = (CamelVeeMessageInfo *) g_hash_table_lookup(((CamelFolderSummary *) s)->loaded_infos, vuid); 
+	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
+
 	if (mi) {
-		g_warning ("%s - already there\n", vuid);
+		/* Possible that the entry is loaded, see if it has the summary */
+		g_message ("%s - already there\n", vuid);
 		g_free (vuid);
+		if (!mi->summary) {
+			mi->summary = summary;
+			camel_object_ref(summary);
+		}
+
 		return mi;
 	}
 
 	mi = (CamelVeeMessageInfo *)camel_message_info_new(&s->summary);
 	mi->summary = summary;
+	/* We would do lazy loading of flags, when the folders are loaded to memory through folder_reloaded signal */
+	mi->old_flags = 0;
 	camel_object_ref (summary);
 	mi->info.uid = (char *) camel_pstring_strdup (vuid);
 	g_free (vuid);
 	camel_message_info_ref (mi);
+	/* Get the flags and store it. We can use it a lot * /
+	rmi = camel_folder_summary_uid (summary, uid);
+	if (rmi) {
+		mi->old_flags = camel_message_info_flags (rmi);
+		camel_message_info_free (rmi);
+	}*/
 	camel_folder_summary_insert(&s->summary, (CamelMessageInfo *)mi, FALSE);
 	
 	return mi;
